@@ -31,11 +31,11 @@
 #include "xparser.h"
 
 //KDE includes
-#include <kdebug.h>
 #include <KLocalizedString>
-#include <kmessagebox.h>
-#include <kconfig.h>
+#include <KMessageBox>
+#include <KConfig>
 
+#include <QDebug>
 #include <QList>
 
 // standard c(++) includes
@@ -81,7 +81,7 @@ ScalarFunction Parser::scalarFunctions[ ScalarCount ]=
 	{"arcsec", "arsec", larcsec},		// Arcus secans = inverse of sec
 	{"arccot", "arcot", larccot},		// Arcus co-tangens = inverse of cotan
 	
-	// Trigometric functions
+	// Trigonometric functions
 	{"sin", QString(), lsin}, 					// Sinus
 	{"cos", QString(), lcos}, 					// Cosinus
 	{"tan", QString(), ltan}, 					// Tangens
@@ -127,7 +127,7 @@ VectorFunction Parser::vectorFunctions[ VectorCount ]=
 
 /**
  * Order by longest string first, useful in parsing since we want to at each point
- * match the longest string first, so e.g. "sinh(x)" should't be read as "sin(h) * x"
+ * match the longest string first, so e.g. "sinh(x)" shouldn't be read as "sin(h) * x"
  */
 class LengthOrderedString : public QString
 {
@@ -329,7 +329,7 @@ double Parser::fkt( Equation * eq, double x )
 			return 0;
 	}
 	
-	kWarning() << "Unknown function type!\n";
+	qWarning() << "Unknown function type!\n";
 	return 0;
 }
 
@@ -355,7 +355,7 @@ double Parser::fkt( Equation * eq, const Vector & x )
 	
 	while(1)
 	{
-// 		kDebug() << "*eq->mptr: "<<int(*eq->mptr);
+// 		qDebug() << "*eq->mptr: "<<int(*eq->mptr);
 		
 		switch(*eq->mptr++)
 		{
@@ -557,7 +557,7 @@ double Parser::fkt( Equation * eq, const Vector & x )
 			{
 				// something went wrong due to a incorrect formular or
 				// missing const.
-				kDebug()  << "Error in equation " << eq->fstr();
+				qDebug()  << "Error in equation " << eq->fstr();
 				// Adjust stack again. Stack is wrong, only if we have a PUSH token
 				// just before the ERROR token, afaik.
 				while( stkptr != stkInitial )
@@ -586,7 +586,7 @@ int Parser::addFunction( const QString & str1, const QString & str2, Function::T
 		int error;
 		if ( !temp->eq[i]->setFstr( str[i], & error, 0, force ) && !force )
 		{
-			kDebug() << "could not set fstr to \""<<str[i]<<"\"!  error:"<<errorString(Error(error))<<"\n";
+			qDebug() << "could not set fstr to \""<<str[i]<<"\"!  error:"<<errorString(Error(error))<<"\n";
 			delete temp;
 			return -1;
 		}
@@ -594,7 +594,7 @@ int Parser::addFunction( const QString & str1, const QString & str2, Function::T
 		bool duplicate = (fnameToID( temp->eq[i]->name() ) != -1);
 		if ( temp->eq[i]->looksLikeFunction() && duplicate && !force )
 		{
-			kDebug() << "function name reused.\n";
+			qDebug() << "function name reused.\n";
 			*m_error = FunctionNameReused;
 			delete temp;
 			return -1;
@@ -602,9 +602,13 @@ int Parser::addFunction( const QString & str1, const QString & str2, Function::T
 	}
 	
 	m_ufkt[ temp->id() ] = temp;
-	
-	temp->plotAppearance( Function::Derivative0 ).color = temp->plotAppearance( Function::Derivative1 ).color = temp->plotAppearance( Function::Derivative2 ).color = temp->plotAppearance( Function::Integral ).color = XParser::self()->defaultColor( temp->id() );
-	
+
+	temp->plotAppearance( Function::Derivative0 ).color = XParser::self()->defaultColor( temp->id() );
+	// Better use QRandomGenerator::global()->generate() instead of qrand() since Qt 5.10
+	temp->plotAppearance( Function::Derivative1 ).color = QColor::fromRgb( qrand() );
+	temp->plotAppearance( Function::Derivative2 ).color = QColor::fromRgb( qrand() );
+	temp->plotAppearance( Function::Integral ).color = QColor::fromRgb( qrand() );
+
 	emit functionAdded( temp->id() );
 	return temp->id(); //return the unique ID-number for the function
 }
@@ -645,7 +649,7 @@ void Parser::initEquation( Equation * eq, Error * error, int * errorPosition )
 	if ( *m_error != ParseSuccess )
 	{
 		*errorPosition = m_sanitizer.realPos( m_evalPos );
-		kDebug() << "add an error token for " << eq->fstr();
+		qDebug() << "add an error token for " << eq->fstr();
 		// add an error token and let the user decide
 		addToken( ERROR );
 	}
@@ -812,7 +816,7 @@ void Parser::heir1()
 					return;
 				}
 				// no break
-				
+				Q_FALLTHROUGH();
 			case '+':
 			case '-':
 				++m_evalPos;
@@ -1577,7 +1581,10 @@ void ExpressionSanitizer::fixExpression( QString * str )
 	}
 	//END replace e.g. |x+2| with abs(x+2)
 	
-	str->replace(m_decimalSymbol, "."); //replace the locale decimal symbol with a '.'
+	if (m_decimalSymbol == ",")
+		str->replace(QRegularExpression("(\\d),(\\d)"), "\\1.\\2"); //if we use comma as a decimal separator it is reasonable to separate numbers in function arguments by comma with space
+	else
+		str->replace(m_decimalSymbol, "."); //replace the locale decimal symbol with a '.' otherwise
 	
 	//BEGIN build up strings
 	QMap< LengthOrderedString, StringType > strings;
@@ -1627,7 +1634,7 @@ void ExpressionSanitizer::fixExpression( QString * str )
 				currentType = NumberString;
 		}
 		
-		if ( (currentType == Other) )
+		if ( currentType == Other )
 		{
 			if ( ch.isLetter() )
 				currentType = UnknownLetter; // probably a variable
@@ -1757,7 +1764,7 @@ int ExpressionSanitizer::realPos( int evalPos )
 	
 	if ( evalPos >= m_map.size() )
 	{
-// 		kWarning() << "evalPos="<<evalPos<<" is out of range.\n";
+// 		qWarning() << "evalPos="<<evalPos<<" is out of range.\n";
 // 		return m_map[ m_map.size() - 1 ];
 		return -1;
 	}
@@ -1778,6 +1785,6 @@ void ExpressionSanitizer::displayMap( )
 		out += "  " + (*m_str)[i];
 	out += '\n';
 	
-	kDebug() << out;
+	qDebug() << out;
 }
 //END class ExpressionSanitizer
